from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from .forms import PageForm
from .models import Page, PageGallery


class PageGalleryInline(admin.StackedInline):
    model = PageGallery
    extra = 0


@admin.register(Page)
class PageAdmin(admin.ModelAdmin):
    form = PageForm
    list_display = ("url", "title")
    list_filter = ("sites", "registration_required")
    search_fields = ("url", "title")
    inlines = [PageGalleryInline]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "url",
                    "title",
                    "thumbnail",
                    "content",
                    "summary",
                    "allow_comments",
                    "registration_required",
                    "sites",
                )
            },
        ),
        (_("SEO Settings"), {"fields": ("template", "seo_title", "seo_description")}),
    )



# Generated by Django 3.2.12 on 2022-03-21 22:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('simpel_pages', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='sites',
            field=models.ManyToManyField(related_name='simpelpages', to='sites.Site', verbose_name='sites'),
        ),
    ]



from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext, gettext_lazy as _

from .models import Page


class PageForm(forms.ModelForm):
    url = forms.RegexField(
        label=_("URL"),
        max_length=100,
        regex=r"^[-\w/\.~]+$",
        help_text=_("Example: “/about/contact/”. Make sure to have leading and trailing slashes."),
        error_messages={
            "invalid": _(
                "This value must contain only letters, numbers, dots, " "underscores, dashes, slashes or tildes."
            ),
        },
    )
    seo_description = forms.CharField(required=False, widget=forms.Textarea(attrs={"rows": 3, "cols": 100}))

    class Meta:
        model = Page
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self._trailing_slash_required():
            self.fields["url"].help_text = _("Example: “/about/contact”. Make sure to have a leading slash.")

    def _trailing_slash_required(self):
        return settings.APPEND_SLASH and "django.middleware.common.CommonMiddleware" in settings.MIDDLEWARE

    def clean_url(self):
        url = self.cleaned_data["url"]
        if not url.startswith("/"):
            raise ValidationError(
                gettext("URL is missing a leading slash."),
                code="missing_leading_slash",
            )
        if self._trailing_slash_required() and not url.endswith("/"):
            raise ValidationError(
                gettext("URL is missing a trailing slash."),
                code="missing_trailing_slash",
            )
        return url

    def clean(self):
        url = self.cleaned_data.get("url")
        sites = self.cleaned_data.get("sites")

        same_url = Page.objects.filter(url=url)
        if self.instance.pk:
            same_url = same_url.exclude(pk=self.instance.pk)

        if sites and same_url.filter(sites__in=sites).exists():
            for site in sites:
                if same_url.filter(sites=site).exists():
                    raise ValidationError(
                        _("SimpelPage with url %(url)s already exists for site %(site)s"),
                        code="duplicate_url",
                        params={"url": url, "site": site},
                    )

        return super().clean()





from django.apps import AppConfig
from django.conf import settings
from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _


class SimpelPagesConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "simpel_pages"
    label = "simpel_pages"
    verbose_name = _("Simpel Pages")

    def ready(self):
        post_migrate.connect(init_app, sender=self)


def init_app(sender, **kwargs):
    """after migrations"""
    from django.contrib.sites.models import Site

    from simpel_themes.models import PathModelTemplate

    from simpel_pages.models import Page

    name = getattr(settings, "SITE_NAME", "Simpel")
    domain = getattr(settings, "SITE_DOMAIN", "127.0.0.1:8000")
    defaults = {"name": name, "domain": domain}
    site, _ = Site.objects.get_or_create(id=getattr(settings, "SITE_ID", 1), defaults=defaults)
    site.name = name
    site.domain = domain
    site.save()

    template, _ = PathModelTemplate.objects.get_or_create(
        template="index.html",
        defaults={
            "name": "Home Page",
        },
    )
    index_page, _ = Page.objects.get_or_create(
        url="/",
        defaults={
            "template": template,
            "title": "Home Page",
        },
    )
    index_page.sites.set([site])



from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import NoReverseMatch, get_script_prefix, reverse
from django.utils.encoding import iri_to_uri
from django.utils.translation import gettext_lazy as _

from ckeditor.fields import RichTextField
from filer.fields.image import FilerImageField
from mptt.models import MPTTModel, TreeForeignKey
from simpel_themes.models import ModelTemplate
from taggit.managers import TaggableManager
from taggit.models import TagBase, TaggedItemBase

from .utils import unique_slugify


class Category(MPTTModel):

    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="children",
        help_text=_(
            "Categories, unlike tags, can have a hierarchy. You might have a "
            "Jazz category, and under that have children categories for Bebop"
            " and Big Band. Totally optional."
        ),
    )
    name = models.CharField(
        max_length=80,
        unique=True,
        verbose_name=_("Category Name"),
    )
    slug = models.SlugField(
        unique=True,
        null=True,
        blank=True,
        editable=False,
        max_length=80,
    )

    icon = "tag-outline"

    class Meta:
        ordering = ["name"]
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")
        permissions = (
            ("import_category", _("Can import Category")),
            ("export_category", _("Can export Category")),
        )

    def __str__(self):
        return self.name

    @property
    def opts(self):
        return self.__class__._meta

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError("Parent category cannot be self.")
            if parent.parent and parent.parent == self:
                raise ValidationError("Cannot have circular Parents.")

    def save(self, *args, **kwargs):
        if not self.slug:
            unique_slugify(self, self.name)
        return super().save(*args, **kwargs)


class Tag(TagBase):

    icon = "tag-outline"

    class Meta:
        verbose_name = _("Tag")
        verbose_name_plural = _("Tags")

    @property
    def opts(self):
        return self._meta


class Page(models.Model):
    url = models.CharField(_("URL"), max_length=100, db_index=True)
    title = models.CharField(_("title"), max_length=200)
    slug = models.SlugField(
        unique=True,
        null=True,
        blank=True,
        editable=False,
        max_length=255,
    )
    thumbnail = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="simpelpages",
    )
    content = RichTextField(_("content"), null=True, blank=True)
    seo_title = models.CharField(_("SEO title"), null=True, blank=True, max_length=200)
    seo_description = models.TextField(_("SEO description"), null=True, blank=True)
    summary = RichTextField(_("summary"), null=True, blank=True)
    template = models.ForeignKey(
        ModelTemplate,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Template"),
    )
    category = models.ForeignKey(
        Category,
        related_name="pages",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Category"),
    )
    tags = TaggableManager(
        through="TaggedPage",
        blank=True,
        related_name="pages",
        verbose_name=_("Tags"),
    )
    allow_comments = models.BooleanField(_("allow comments"), default=False)
    registration_required = models.BooleanField(
        _("registration required"),
        help_text=_("If this is checked, only logged-in users will be able to view the page."),
        default=False,
    )
    sites = models.ManyToManyField(Site, verbose_name=_("sites"), related_name="simpelpages")

    class Meta:
        verbose_name = _("page")
        verbose_name_plural = _("pages")
        ordering = ["url"]

    def __str__(self):
        return "%s -- %s" % (self.url, self.title)

    def get_absolute_url(self):
        from .views import simpelpage

        for url in (self.url.lstrip("/"), self.url):
            try:
                return reverse(simpelpage, kwargs={"url": url})
            except NoReverseMatch:
                pass
        # Handle script prefix manually because we bypass reverse()
        return iri_to_uri(get_script_prefix().rstrip("/") + self.url)

    def save(self, *args, **kwargs):
        if not self.slug:
            unique_slugify(self, self.title)
        return super().save(*args, **kwargs)


class TaggedPage(TaggedItemBase):
    class Meta:
        db_table = "simpel_product_tagged"
        verbose_name = _("Tagged Product")
        verbose_name_plural = _("Tagged Products")

    content_object = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        related_name="tagged_pages",
        db_index=True,
    )
    tag = models.ForeignKey(
        Tag,
        on_delete=models.CASCADE,
        related_name="tagged_pages",
        db_index=True,
    )

    def __str__(self):
        return str(self.tag)


class PageGallery(models.Model):
    page = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        related_query_name="galleries",
        verbose_name=_("page"),
    )
    caption = models.CharField(
        max_length=200,
        verbose_name=_("Caption"),
    )
    thumb_height = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Height"),
    )
    thumb_width = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Width"),
    )
    image = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="simpelpages_gallery_items",
    )

    class Meta:
        verbose_name = _("Image Gallery")
        verbose_name_plural = _("Image Galleries")
        index_together = ("page", "image")
        ordering = ["page"]
